Newtype

Trait Newtype 

pub trait Newtype {
    type Inner;

    // Required methods
    fn new(inner: Self::Inner) -> Self;
    fn into_inner(self) -> Self::Inner;
}
Newtype trait defines conversions from and into the inner type.

This trait is automatically derived for all types annotated with #[derive(Newtype)].

Required Associated Types

type Inner

The inner type.

Required Methods

fn new(inner: Self::Inner) -> Self

Creates a new newtype instance from the inner representation.

fn into_inner(self) -> Self::Inner

Unwraps the value, consuming the newtype.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors